@@ -557,13 +557,13 @@ wrapped_gobj_toggle_notify(gpointer      data,
 
     runtime = data;
 
-    /* The JSContext will be gone if runtime is being destroyed.
+    /* During teardown, this can return NULL if runtime is being destroyed.
      * In that case we effectively already converted to a weak ref without
      * doing anything since the keep alive will be collected.
      * Or if !is_last_ref, we do not want to convert to a strong
      * ref since we want everything collected on runtime destroy.
      */
-    context = gjs_runtime_peek_load_context(runtime);
+    context = gjs_runtime_get_current_context(runtime);
     if (!context)
         return;
 
@@ -595,7 +595,7 @@ wrapped_gobj_toggle_notify(gpointer      data,
          */
         if (priv->keep_alive == NULL) {
             gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "Adding object to keep alive");
-            priv->keep_alive = gjs_keep_alive_get_for_load_context(runtime);
+            priv->keep_alive = gjs_keep_alive_get_for_import_global(context);
             gjs_keep_alive_add_child(context, priv->keep_alive,
                                      gobj_no_longer_kept_alive_func,
                                      obj,
@@ -743,7 +743,7 @@ object_instance_constructor(JSContext *context,
          * the wrapper to be garbage collected (and thus unref the
          * wrappee).
          */
-        priv->keep_alive = gjs_keep_alive_get_for_load_context(JS_GetRuntime(context));
+        priv->keep_alive = gjs_keep_alive_get_for_import_global(context);
         gjs_keep_alive_add_child(context,
                                  priv->keep_alive,
                                  gobj_no_longer_kept_alive_func,
