@@ -285,11 +285,16 @@ load_module_init(JSContext  *context,
         }
     }
 
-    module_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    module_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (module_obj == NULL) {
         return JS_FALSE;
     }
 
+    /* https://bugzilla.mozilla.org/show_bug.cgi?id=599651 means we
+     * can't just pass in the global as the parent */
+    JS_SetParent(context, module_obj,
+                 gjs_get_import_global (context));
+
     /* Define module in importer for future use and to avoid module_obj
      * object to be garbage collected during the evaluation of the script */
     JS_DefineProperty(context, in_object,
@@ -724,6 +729,9 @@ importer_new_enumerate(JSContext  *context,
     ImporterIterator *iter;
 
     switch (enum_op) {
+#ifdef HAVE_JSENUMERATE_INIT_ALL
+    case JSENUMERATE_INIT_ALL:
+#endif
     case JSENUMERATE_INIT: {
         Importer *priv;
         JSObject *search_path;
@@ -735,7 +743,7 @@ importer_new_enumerate(JSContext  *context,
             *state_p = JSVAL_NULL;
 
         if (id_p)
-            *id_p = JSID_VOID;
+            *id_p = INT_TO_JSID(0);
 
         priv = priv_from_js(context, object);
         if (!priv)
@@ -842,7 +850,7 @@ importer_new_enumerate(JSContext  *context,
             *state_p = PRIVATE_TO_JSVAL(iter);
 
         if (id_p)
-            *id_p = INT_TO_JSVAL(iter->elements->len);
+            *id_p = INT_TO_JSID(iter->elements->len);
 
         break;
     }
@@ -855,6 +863,9 @@ importer_new_enumerate(JSContext  *context,
             return JS_FALSE;
         }
 
+        if (*state_p == JSVAL_NULL) /* Iterating prototype */
+            return JS_TRUE;
+
         iter = JSVAL_TO_PRIVATE(*state_p);
 
         if (iter->index < iter->elements->len) {
@@ -909,7 +920,6 @@ importer_new_resolve(JSContext *context,
 {
     Importer *priv;
     const char *name;
-    JSContext *load_context;
 
     *objp = NULL;
 
@@ -928,21 +938,13 @@ importer_new_resolve(JSContext *context,
     if (priv == NULL)
         return JS_TRUE; /* we are the prototype, or have the wrong class */
 
-    /* We always import in the special load context. */
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
-    if (do_import(load_context, obj, priv, name)) {
+    JS_BeginRequest(context);
+    if (do_import(context, obj, priv, name)) {
         *objp = obj;
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return JS_TRUE;
     } else {
-        /* Move the exception to the calling context from load context.
-         */
-        if (!gjs_move_exception(load_context, context)) {
-            /* set an exception since none was set */
-            gjs_throw(context, "No exception was set, but import failed somehow");
-        }
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return JS_FALSE;
     }
 }
@@ -1034,7 +1036,7 @@ importer_new(JSContext    *context)
     Importer *priv;
     JSObject *global;
 
-    global = JS_GetGlobalObject(context);
+    global = gjs_get_import_global(context);
 
     if (!gjs_object_has_property(context, global, gjs_importer_class.name)) {
         JSObject *prototype;
@@ -1069,7 +1071,7 @@ importer_new(JSContext    *context)
                   gjs_importer_class.name, prototype);
     }
 
-    importer = JS_ConstructObject(context, &gjs_importer_class, NULL, NULL);
+    importer = JS_ConstructObject(context, &gjs_importer_class, NULL, global);
     if (importer == NULL)
         gjs_fatal("No memory to create ns object");
 
@@ -1182,20 +1184,20 @@ gjs_define_importer(JSContext    *context,
  * we just ignore all calls after the first and hope the args are the same.
  */
 JSBool
-gjs_create_root_importer(JSRuntime   *runtime,
+gjs_create_root_importer(JSContext   *context,
                          const char **initial_search_path,
                          gboolean     add_standard_search_path)
 {
-    JSContext *context;
+    JSObject *global;
 
-    context = gjs_runtime_get_load_context(runtime);
+    global = gjs_get_import_global(context);
 
     JS_BeginRequest(context);
 
     if (!gjs_object_has_property(context,
-                                 JS_GetGlobalObject(context),
+                                 global,
                                  "imports")) {
-        if (gjs_define_importer(context, JS_GetGlobalObject(context),
+        if (gjs_define_importer(context, global,
                                 "imports",
                                 initial_search_path, add_standard_search_path) == NULL) {
             JS_EndRequest(context);
@@ -1217,16 +1219,16 @@ gjs_define_root_importer(JSContext   *context,
                          JSObject    *in_object,
                          const char  *importer_name)
 {
-    JSContext *load_context;
+    JSObject *global;
     jsval value;
     JSBool success;
 
     success = JS_FALSE;
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    global = gjs_get_import_global(context);
+    JS_BeginRequest(context);
 
-    if (!gjs_object_require_property(load_context,
-                                     JS_GetGlobalObject(load_context), "global object",
+    if (!gjs_object_require_property(context,
+                                     global, "global object",
                                      "imports", &value) ||
         !JSVAL_IS_OBJECT(value)) {
         gjs_debug(GJS_DEBUG_IMPORTER, "Root importer did not exist, couldn't get from load context; must create it");
@@ -1244,6 +1246,6 @@ gjs_define_root_importer(JSContext   *context,
 
     success = JS_TRUE;
  fail:
-    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return success;
 }
