@@ -24,6 +24,7 @@
 #include <config.h>
 
 #include "jsapi-util.h"
+#include "compat.h"
 
 /* Maximum number of elements allowed in a GArray of rooted jsvals.
  * We pre-alloc that amount and then never allow the array to grow,
@@ -59,7 +60,9 @@ static void
 add_root_jsval(JSContext *context,
                jsval     *value_p)
 {
-    JS_AddRoot(context, value_p);
+    JS_BeginRequest(context);
+    JS_AddValueRoot(context, value_p);
+    JS_EndRequest(context);
 }
 
 /* typesafe wrapper */
@@ -67,7 +70,9 @@ static void
 remove_root_jsval(JSContext *context,
                   jsval     *value_p)
 {
-    JS_RemoveRoot(context, value_p);
+    JS_BeginRequest(context);
+    JS_RemoveValueRoot(context, value_p);
+    JS_EndRequest(context);
 }
 
 /**
@@ -76,7 +81,7 @@ remove_root_jsval(JSContext *context,
  * @array: a #GjsRootedArray created by gjs_rooted_array_new()
  * @value: a jsval
  *
- * Appends @jsval to @array, calling JS_AddRoot on the location where it's stored.
+ * Appends @jsval to @array, calling JS_AddValueRoot on the location where it's stored.
  *
  **/
 void
@@ -176,7 +181,7 @@ gjs_rooted_array_get_length (JSContext        *context,
  * @locations: contiguous locations in memory that store jsvals (must be initialized)
  * @n_locations: the number of locations to root
  *
- * Calls JS_AddRoot() on each address in @locations.
+ * Calls JS_AddValueRoot() on each address in @locations.
  *
  **/
 void
@@ -190,9 +195,11 @@ gjs_root_value_locations(JSContext        *context,
     g_return_if_fail(locations != NULL);
     g_return_if_fail(n_locations >= 0);
 
+    JS_BeginRequest(context);
     for (i = 0; i < n_locations; i++) {
         add_root_jsval(context, ((jsval*)locations) + i);
     }
+    JS_EndRequest(context);
 }
 
 /**
@@ -201,7 +208,7 @@ gjs_root_value_locations(JSContext        *context,
  * @locations: contiguous locations in memory that store jsvals and have been added as GC roots
  * @n_locations: the number of locations to unroot
  *
- * Calls JS_RemoveRoot() on each address in @locations.
+ * Calls JS_RemoveValueRoot() on each address in @locations.
  *
  **/
 void
@@ -215,9 +222,11 @@ gjs_unroot_value_locations(JSContext *context,
     g_return_if_fail(locations != NULL);
     g_return_if_fail(n_locations >= 0);
 
+    JS_BeginRequest(context);
     for (i = 0; i < n_locations; i++) {
         remove_root_jsval(context, ((jsval*)locations) + i);
     }
+    JS_EndRequest(context);
 }
 
 /**
@@ -278,35 +287,23 @@ gjs_rooted_array_free(JSContext        *context,
 }
 
 #if GJS_BUILD_TESTS
+#include "unit-test-utils.h"
 #include <string.h>
 
-static void
-test_error_reporter(JSContext     *context,
-                    const char    *message,
-                    JSErrorReport *report)
-{
-    g_printerr("error reported by test: %s\n", message);
-}
-
 #define N_ELEMS 15
 
 void
 gjstest_test_func_gjs_jsapi_util_array(void)
 {
-    JSRuntime *runtime;
+    GjsUnitTestFixture fixture;
     JSContext *context;
     JSObject *global;
     GjsRootedArray *array;
     int i;
     jsval value;
 
-    runtime = JS_NewRuntime(1024*1024 /* max bytes */);
-    context = JS_NewContext(runtime, 8192);
-    global = JS_NewObject(context, NULL, NULL, NULL);
-    JS_SetGlobalObject(context, global);
-    JS_InitStandardClasses(context, global);
-
-    JS_SetErrorReporter(context, test_error_reporter);
+    _gjs_unit_test_fixture_begin(&fixture);
+    context = fixture.context;
 
     array = gjs_rooted_array_new();
 
@@ -332,9 +329,7 @@ gjstest_test_func_gjs_jsapi_util_array(void)
 
     gjs_rooted_array_free(context, array, TRUE);
 
-    JS_DestroyContext(context);
-    JS_DestroyRuntime(runtime);
-    JS_ShutDown();
+    _gjs_unit_test_fixture_finish(&fixture);
 }
 
 #endif /* GJS_BUILD_TESTS */
