@@ -278,6 +278,18 @@ complete_call(JSContext   *context,
     return JS_TRUE;
 }
 
+static JSContext *
+get_callback_context(GClosure *closure)
+{
+    JSRuntime *runtime;
+
+    if (!gjs_closure_is_valid(closure))
+        return NULL;
+
+    runtime = gjs_closure_get_runtime(closure);
+    return gjs_runtime_get_current_context(runtime);
+}
+
 static void
 pending_notify(DBusPendingCall *pending,
                void            *user_data)
@@ -291,7 +303,7 @@ pending_notify(DBusPendingCall *pending,
 
     closure = user_data;
 
-    context = gjs_closure_get_context(closure);
+    context = get_callback_context(closure);
 
     gjs_debug(GJS_DEBUG_DBUS,
               "Notified of reply to async call closure %p context %p",
@@ -636,7 +648,7 @@ signal_handler_callback(DBusConnection *connection,
         return;
     }
 
-    context = gjs_closure_get_context(handler->closure);
+    context = get_callback_context(handler->closure);
 
     if (!context) {
         /* The runtime is gone */
@@ -1007,7 +1019,7 @@ on_name_acquired(DBusConnection *connection,
 
     owner = data;
 
-    context = gjs_closure_get_context(owner->acquired_closure);
+    context = get_callback_context(owner->acquired_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name acquired");
@@ -1046,7 +1058,7 @@ on_name_lost(DBusConnection *connection,
 
     owner = data;
 
-    context = gjs_closure_get_context(owner->lost_closure);
+    context = get_callback_context(owner->lost_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name lost");
@@ -1227,7 +1239,7 @@ on_name_appeared(DBusConnection *connection,
 
     watcher = data;
 
-    context = gjs_closure_get_context(watcher->appeared_closure);
+    context = get_callback_context(watcher->appeared_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name appeared");
@@ -1270,7 +1282,7 @@ on_name_vanished(DBusConnection *connection,
 
     watcher = data;
 
-    context = gjs_closure_get_context(watcher->vanished_closure);
+    context = get_callback_context(watcher->vanished_closure);
     if (context == NULL) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Closure destroyed before we could notify name vanished");
@@ -1738,9 +1750,16 @@ define_bus_object(JSContext  *context,
     bus_val = JSVAL_VOID;
     JS_AddValueRoot(context, &bus_val);
 
-    bus_obj = JS_ConstructObject(context, NULL, proto_obj, NULL);
+    bus_obj = JS_ConstructObject(context, NULL, NULL, NULL);
     if (bus_obj == NULL)
         goto out;
+    /* We need to use a separate call to SetPrototype to work
+     * around a SpiderMonkey bug where with clasp=NULL, the
+     * parent and proto arguments to JS_ConstructObject are
+     * lost.
+     * https://bugzilla.mozilla.org/show_bug.cgi?id=599651
+     */
+    JS_SetPrototype(context, bus_obj, proto_obj);
 
     bus_val = OBJECT_TO_JSVAL(bus_obj);
 
