@@ -89,7 +89,8 @@ test(GjsTestJSFixture *fix,
     if (!success)
         g_error("%s", error->message);
     g_assert(error == NULL);
-    g_assert_cmpint(code, ==, 0);
+    if (code != 0)
+        g_error("Test script returned code %d; assertions will be in gjs.log", code);
 }
 
 static GSList *
@@ -174,7 +175,6 @@ main(int argc, char **argv)
     g_setenv("BUILDDIR", top_builddir, FALSE);
     g_setenv("XDG_DATA_HOME", data_home, FALSE);
     g_setenv("GJS_PATH", path->str, FALSE);
-
     /* The tests are known to fail in the presence of the JIT;
      * we leak objects.
      * https://bugzilla.gnome.org/show_bug.cgi?id=616193
